#!/usr/bin/env node
import apolloClient from '@apollo/client';
import fs from 'fs';
import yargs from 'yargs';
import { hideBin } from 'yargs/helpers';
import { stripSymbols } from 'apollo-utilities';
import moment from "moment";
const { ApolloClient, InMemoryCache, gql } = apolloClient;
let current_time = Date.now();

function createApolloClient(uri, token) {
    return new ApolloClient({
        uri,
        cache: new InMemoryCache(),
        headers: {
            Authorization: `Bearer ${token}`,
        },
    });
}
async function getMigrationsEndId(client) {
    const result = await client.query({
        query: gql`
            query Links {
                links(where: {type_id: {_eq: "182"}}) {
                    id
                }
            }
        `
    });
    return result.data.links[0].id;
}
function getLinksGreaterThanId(client, id) {
    return client.query({
        query: gql`query ExportLinks {
            links(order_by: { id: asc }, where: { id: { _gte: ${id} } }) {
                id
                from_id
                to_id
                type_id
                object {
                    value
                }
                string {
                    value
                }
                number {
                    value
                }
            }
        }
        `,
    })
}
async function exportData(url, jwt, filename  = `./Dumps/dump-${moment(current_time).format("YYYY-MM-DD-HH-mm-ss")}.json`) {
    const client = createApolloClient(url, jwt)
    getLinksGreaterThanId(client, await getMigrationsEndId(client))
        .then((result) => {
            let links = stripSymbols(result)
            links = links.data.links.slice()
            for (let item of links) {
                if (item?.object?.__typename) {
                    delete item.object.__typename;
                }
                if (item?.string?.__typename) {
                    delete item.string.__typename;
                }
                if (item?.number?.__typename) {
                    delete item.number.__typename;
                }
                if (item?.__typename){
                    delete item.__typename
                }
            }

            fs.writeFileSync(filename, JSON.stringify(links), (err) => {
                if (err) throw err;
                console.log('File saved!');
            });

            console.log(result.data)
        })
        .catch((error) => console.error(error));
}

yargs(hideBin(process.argv))
    .command('deep-export', '', (yargs) => {
        return yargs
            .option('url', { describe: 'The url to export data from', type: 'string', demandOption: true })
            .option('jwt', { describe: 'The JWT token', type: 'string', demandOption: true })
            .option('file', { describe: 'The file to save data to', type: 'string', demandOption: false });

    }, (argv) => {
        console.log(argv.url)
        console.log(argv.jwt)
        console.log(argv.file)
        exportData(argv.url, argv.jwt, argv.file)
            .catch((error) => console.error(error));
    })
    .demandCommand(1, 'You need at least one command before moving on')
    .help()
    .argv;
